"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExtensionServerSettings = void 0;
const dntShim = __importStar(require("../../../_dnt.shims.js"));
const addresses_js_1 = require("../../core-validators/addresses.js");
const crypto_js_1 = require("../../crypto/crypto.js");
const query_helpers_js_1 = require("../../query/query-helpers.js");
const errors_js_1 = require("../../util/errors.js");
const HOSTED_SHARE_TEMPLATE = `/server-settings/1.*/shares/{shareHash}/{hostType}`;
/** A server extension which reads settings from a specified share, e.g. which shares to host on the server. Settings are modified by other peers syncing with this server. */
class ExtensionServerSettings {
    constructor(opts) {
        Object.defineProperty(this, "configReplica", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "onCreateReplica", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.configReplica = opts.onCreateReplica(opts.settingsShare);
        this.onCreateReplica = opts.onCreateReplica;
    }
    register(peer) {
        peer.addReplica(this.configReplica);
        const onCreateReplica = this.onCreateReplica;
        const { glob } = (0, query_helpers_js_1.parseTemplate)(HOSTED_SHARE_TEMPLATE);
        const { query, regex } = (0, query_helpers_js_1.globToQueryAndRegex)(glob);
        const configShareAddress = this.configReplica.share;
        this.configReplica.getQueryStream(query, "everything").pipeTo(new dntShim.WritableStream({
            async write(event) {
                if (event.kind === "existing" || event.kind === "success") {
                    if (regex != null &&
                        new RegExp(regex).test(event.doc.path)) {
                        const pathVariables = (0, query_helpers_js_1.extractTemplateVariablesFromPath)(HOSTED_SHARE_TEMPLATE, event.doc.path);
                        if (!pathVariables) {
                            return;
                        }
                        const shareHash = pathVariables["shareHash"];
                        const configHash = await crypto_js_1.Crypto.sha256base32(configShareAddress);
                        if (shareHash === configHash) {
                            return;
                        }
                        if (event.doc.text.length > 0 &&
                            (0, errors_js_1.notErr)((0, addresses_js_1.parseShareAddress)(event.doc.text)) &&
                            !peer.hasShare(event.doc.text)) {
                            // Add
                            const replica = onCreateReplica(event.doc.text);
                            peer.addReplica(replica);
                            console.log("Server settings:", `now hosting ${replica.share}`);
                        }
                        else if (event.doc.text.length === 0) {
                            let replicaToClose;
                            for (const replica of peer.replicas()) {
                                const rShareHash = await crypto_js_1.Crypto.sha256base32(replica.share);
                                if (rShareHash === shareHash) {
                                    replicaToClose = replica;
                                }
                            }
                            if (replicaToClose) {
                                await replicaToClose.close(true);
                                peer.removeReplica(replicaToClose);
                                console.log("Server settings:", `stopped hosting ${replicaToClose.share}`);
                            }
                        }
                    }
                }
                if (event.kind === "expire") {
                    if (regex != null &&
                        new RegExp(regex).test(event.doc.path)) {
                        // Remove
                        const pathVariables = (0, query_helpers_js_1.extractTemplateVariablesFromPath)(HOSTED_SHARE_TEMPLATE, event.doc.path);
                        if (!pathVariables) {
                            return;
                        }
                        const shareAddress = pathVariables["shareAddress"];
                        const replicaToClose = peer.getReplica(shareAddress);
                        if (replicaToClose) {
                            await replicaToClose.close(true);
                            peer.removeReplica(replicaToClose);
                            console.log("Server settings:", `stopped hosting ${replicaToClose.share}`);
                        }
                    }
                }
            },
        }));
        return Promise.resolve();
    }
    handler(_req) {
        return Promise.resolve(null);
    }
}
exports.ExtensionServerSettings = ExtensionServerSettings;
